@@ -387,12 +387,11 @@ def main(args=None):
 
     if env_vmname:
         vm = args.app.domains[env_vmname]
+    elif not args.domains:
+        parser.error("You must specify at least the VM name!")
     else:
         vm = args.domains[0]
 
-    if vm is None:
-        parser.error("You must specify at least the VM name!")
-
     if env_vmname is None or args.force_rpc:
         use_stdin = False
     else:
@@ -400,7 +399,7 @@ def main(args=None):
     appmenusext = qubesappmenus.Appmenus()
     if not args.regenerate_only:
         new_appmenus = retrieve_appmenus_templates(vm, use_stdin=use_stdin)
-        if not new_appmenus:
+        if not new_appmenus and vm.klass != "AppVM":
             vm.log.info("No appmenus received, terminating")
         else:
             process_appmenus_templates(appmenusext, vm, new_appmenus)
